use crate::app_config::AppConfig;
use crate::extractors::money_session::MoneySession;
use crate::models::tokens::Token;
use crate::models::users::{User, UserID};
use crate::services::users_service;
use actix_web::dev::Payload;
use actix_web::error::ErrorPreconditionFailed;
use actix_web::{Error, FromRequest, HttpRequest};

#[derive(Debug, Clone)]
pub enum AuthenticatedMethod {
    /// User is authenticated using a cookie
    Cookie,
    /// User is authenticated through command line, for debugging purposes only
    Dev,
    // TODO : token implementation
    Token(Token),
}

/// Authentication extractor. Extract authentication information from request
pub struct AuthExtractor {
    pub method: AuthenticatedMethod,
    pub user: User,
}

impl AuthExtractor {
    /// Get current user ID
    pub fn user_id(&self) -> UserID {
        self.user.id()
    }
}

impl FromRequest for AuthExtractor {
    type Error = Error;
    type Future = futures_util::future::LocalBoxFuture<'static, Result<Self, Self::Error>>;

    fn from_request(req: &HttpRequest, _payload: &mut Payload) -> Self::Future {
        let req = req.clone();
        Box::pin(async move {
            // Check if login is hard-coded as program argument
            if let Some(email) = &AppConfig::get().unsecure_auto_login_email {
                let user = users_service::get_user_by_email(email).map_err(|e| {
                    log::error!("Failed to retrieve dev user: {e}");
                    ErrorPreconditionFailed("Unable to retrieve dev user!")
                })?;
                return Ok(Self {
                    method: AuthenticatedMethod::Dev,
                    user,
                });
            }

            // Check for cookie authentication
            let session = MoneySession::extract(&req).await?;
            if let Some(user_id) = session.current_user().map_err(|e| {
                log::error!("Failed to retrieve user id: {e}");
                ErrorPreconditionFailed("Failed to read session information!")
            })? {
                let user = users_service::get_user_by_id(user_id).map_err(|e| {
                    log::error!("Failed to retrieve user from cookie session: {e}");
                    ErrorPreconditionFailed("Failed to retrieve user information!")
                })?;
                return Ok(Self {
                    method: AuthenticatedMethod::Cookie,
                    user,
                });
            };

            Err(ErrorPreconditionFailed("Authentication required!"))
        })
    }
}
